local _, class = UnitClass("player")
if class ~= "DEATHKNIGHT" then
	return
end

local plugin = mRunes:NewModule("Diseases", "AceEvent-3.0")
plugin.mRunesPlugin = true

local AceGUI = LibStub("AceGUI-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local BD = LibStub("LibBackdrop-1.0")
local UIF
local db

local LUnitDebuff = UnitDebuff

local InCombat = false
local HaveRP = false

local FrostName, _, FrostIcon =  GetSpellInfo(59921)
local PlagueName, _, PlagueIcon =  GetSpellInfo(59879)

local backdrop = {
	tile = true, 
	tileSize = 16,
	insets = {},
}

function plugin:OnInitialize()
	local defaults = {
		profile = {
			Enabled = true,
			x = 0,
			y = -60,
			Width = 337,
			Height = 20,			
			InternalSpacing = 1,	
			BarSpacing = 2,
			
			AlphaIncCombat = 1,
			AlphaOutCombat = 0.2,
			AlphaActive = 0.5,
			
			HorAlign = "RIGHT",
			VerAlign = "BOTTOM",
			AlwaysShowText = false,
			ShowText = true,
			
			FrostColor = {0.0, 1.0, 1.0, 1},
			PlagueColor = {0.8, 0.1, 1.0, 1},
			
			Font = "accid",
			FontSize = 16,
			FontFlags = "OUTLINE",
			FontColor = {1, 1, 1, 1},
			
			Texture = "HalD",
			
			BorderTexture = "None",
			BorderWidth = 5,
			BorderInsets = 0,
			BorderPadding = 0,
			BorderColor = {0, 0, 0, 1},
			BackgroundColor = {0, 0, 0, 0.8},
		}
	}
	
	self.db = mRunes.db:RegisterNamespace(self:GetName(), defaults)
	self.db.RegisterCallback(self, "OnProfileChanged", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileCopied", "ProfileChanged")
	self.db.RegisterCallback(self, "OnProfileReset", "ProfileChanged")
	db = self.db.profile
	
	self:SetEnabledState(db.Enabled)
		
	UIF = LibStub("LibGUIFactory-1.0"):GetFactory("mRunes")
end

function plugin:OnEnable()
	if not self.DiseaseFrame then
		self:CreateDiseaseFrame()
	end
	
	self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	mRunes.RegisterCallback(self, "EnteringCombat")
	mRunes.RegisterCallback(self, "LeavingCombat")
	
	self:ApplySettings()
		
	self:UNIT_AURA("UNIT_AURA", "target")
	
	if UnitExists("target") and  UnitReaction("player", "target") then
		mRunes.FadeIn(self.DiseaseFrame, false, db.AlphaIncCombat)
	else
		mRunes.FadeOut(self.DiseaseFrame, db.AlphaOutCombat)
	end
end

function plugin:OnDisable()
	mRunes:UnregisterOnUpdate(self.DiseaseFrame, self.Update)
	if self.DiseaseFrame then mRunes.FadeOut(self.DiseaseFrame) end
end

function plugin:ProfileChanged(event)
	db = self.profile.db
	self:ApplySettings()
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Callbacks ~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:EnteringCombat(event)
	if UnitExists("target") and  UnitReaction("player", "target") then
		mRunes.FadeIn(self.DiseaseFrame, false, db.AlphaIncCombat)
	end
	InCombat = true
end

function plugin:LeavingCombat(event)
	mRunes.FadeOut(self.DiseaseFrame, db.AlphaOutCombat)
	InCombat = false
end

function plugin:EnterWorld(...)
end

function plugin:Update()
	local t = GetTime()
	local bar = plugin.DiseaseFrame.FrostBar
	if bar.duration then
		local dur = bar.expires - t
		bar:SetValue(dur / bar.duration)
		if db.ShowText then
			bar.text:SetFormattedText("%.1f", dur)
		end
	end
	
	local bar = plugin.DiseaseFrame.PlagueBar
	if bar.duration then
		local dur = bar.expires - t
		bar:SetValue(dur / bar.duration)
		if db.ShowText then
			bar.text:SetFormattedText("%.1f", dur)
		end
	end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Events ~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:UNIT_AURA(event, unit)	
	local DiseaseFrame = self.DiseaseFrame
	if DiseaseFrame then
		if unit == "target" then
			DiseaseFrame.DoUpdate = false
			local name, rank, icon, count, atype, duration, expires, caster, stealable, shouldConsolidate, spellId  = LUnitDebuff(unit, FrostName)
			
			local bar = DiseaseFrame.FrostBar
			if name and caster == "player" then								
				local bar = DiseaseFrame.FrostBar
				bar.expires = expires
				bar.duration = duration
				DiseaseFrame.DoUpdate = true
			else
				bar.expires = nil
				bar.duration = nil
				bar:SetValue(0)				
				if not db.AlwaysShowText then
					bar.text:SetText()
				else
					bar.text:SetText(0)
				end
			end
			
			name, rank, icon, count, atype, duration, expires, caster, stealable, shouldConsolidate, spellId  = LUnitDebuff(unit, PlagueName)				
			local bar = DiseaseFrame.PlagueBar			
			if name  and caster == "player" then	
				bar.expires = expires
				bar.duration = duration
				DiseaseFrame.DoUpdate = true
			else
				bar.expires = nil
				bar.duration = nil
				bar:SetValue(0)
				if not db.AlwaysShowText then
					bar.text:SetText()
				else
					bar.text:SetText(0)
				end			
			end
			
			if DiseaseFrame.DoUpdate then
				mRunes:RegisterOnUpdate(self.DiseaseFrame, self.Update)
			else
				mRunes:UnregisterOnUpdate(self.DiseaseFrame, self.Update)
			end
		end
	end
end

function plugin:PLAYER_TARGET_CHANGED(event)
	if UnitExists("target") and  UnitReaction("player", "target") < 5 and UnitAffectingCombat("player") then
		self.DiseaseFrame:SetAlpha(db.AlphaIncCombat)
	else
		self.DiseaseFrame:SetAlpha(db.AlphaOutCombat)
	end
	self:UNIT_AURA("UNIT_AURA", "target")
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Settings	~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:ApplySettings()
	local DiseaseFrame = self.DiseaseFrame
	if DiseaseFrame then
		DiseaseFrame:ClearAllPoints()
		DiseaseFrame:SetPoint("CENTER", mRunes.MainFrame, "CENTER", db.x, db.y)
		DiseaseFrame:SetWidth(db.Width)
		DiseaseFrame:SetHeight(db.Height)
		
		backdrop.insets.left = db.BorderInsets
		backdrop.insets.right = db.BorderInsets
		backdrop.insets.top = db.BorderInsets
		backdrop.insets.bottom = db.BorderInsets
		backdrop.edgeFile = LSM:Fetch("border", db.BorderTexture)
		backdrop.edgeSize = db.BorderWidth
		backdrop.bgFile = LSM:Fetch("background", "Solid")
		DiseaseFrame:SetBackdrop(backdrop)
		DiseaseFrame:SetBackdropColor(unpack(db.BackgroundColor))
		DiseaseFrame:SetBackdropBorderColor(unpack(db.BorderColor))
		
		local bar = DiseaseFrame.FrostBar
		bar:ClearAllPoints()
		bar:SetPoint("TOPLEFT", db.BorderPadding, -db.BorderPadding)
		bar:SetPoint("TOPRIGHT", -db.BorderPadding, -db.BorderPadding)
		bar:SetHeight(db.Height / 2 - db.BorderPadding - db.BarSpacing / 2)
		
		bar:SetStatusBarTexture(LSM:Fetch("statusbar", db.Texture))
		bar:GetStatusBarTexture():SetHorizTile(false)
		bar:SetStatusBarColor(unpack(db.FrostColor))
		
		bar.text:SetPoint("TOPLEFT", db.InternalSpacing, -db.InternalSpacing)
		bar.text:SetPoint("BOTTOMRIGHT", -db.InternalSpacing, db.InternalSpacing)
		bar.text:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
		bar.text:SetTextColor(unpack(db.FontColor))
		bar.text:SetJustifyH(db.HorAlign)
		bar.text:SetJustifyV(db.VerAlign)
		
		local bar = DiseaseFrame.PlagueBar
		bar:ClearAllPoints()
		bar:SetPoint("BOTTOMLEFT", db.BorderPadding, db.BorderPadding)		
		bar:SetPoint("BOTTOMRIGHT", -db.BorderPadding, db.BorderPadding)
		bar:SetHeight(db.Height / 2  - db.BorderPadding - db.BarSpacing / 2)
		bar:SetStatusBarTexture(LSM:Fetch("statusbar", db.Texture))
		bar:GetStatusBarTexture():SetHorizTile(false)
		bar:SetStatusBarColor(unpack(db.PlagueColor))
		
		bar.text:SetPoint("TOPLEFT", db.InternalSpacing, -db.InternalSpacing)
		bar.text:SetPoint("BOTTOMRIGHT", -db.InternalSpacing, db.InternalSpacing)
		bar.text:SetFont(LSM:Fetch("font", db.Font), db.FontSize, db.FontFlags)
		bar.text:SetTextColor(unpack(db.FontColor))
		bar.text:SetJustifyH(db.HorAlign)
		bar.text:SetJustifyV(db.VerAlign)
		
		DiseaseFrame:SetAlpha(InCombat and db.AlphaIncCombat or (HaveRP and db.AlphaActive or db.AlphaOutCombat))
		if DiseaseFrame:GetAlpha() > 0 then DiseaseFrame:Show() else DiseaseFrame:Hide() end		
		--[[if db.AlwaysShowText and db.ShowText then
			PowerFrame.text:Show()
		else
			PowerFrame.text:Hide()				
		end]]--
	
	end	
	
	if not db.Enabled and self:IsEnabled() then self:Disable()
	elseif db.Enabled and not self:IsEnabled() then self:Enable() end
end

-- ~~~~~~~~~~~~~~~~~~~~
-- Create Frames ~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~
function plugin:CreateDiseaseFrame()
	local frame = CreateFrame("Frame", nil, UIParent)
	
	BD:EnhanceBackdrop(frame)
	
	local bar = CreateFrame("StatusBar", "mRunesFrostFeverBar", frame)
	bar:SetMinMaxValues(0,1)
	bar:SetValue(0)
	frame.FrostBar = bar
	local text = bar:CreateFontString(nil, nil, "GameFontNormal")
	bar.text = text	
	
	local bar = CreateFrame("StatusBar", "mRunesPlaugeBar", frame)
	bar:SetMinMaxValues(0,1)
	bar:SetValue(0)
	frame.PlagueBar = bar	
	local text = bar:CreateFontString(nil, nil, "GameFontNormal")
	bar.text = text	
	
	self.DiseaseFrame = frame
end

-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
-- Options ~~~~~~~~~~~~~~~~~~~~
-- ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
function plugin:GetOptions()
	return plugin.GetGUI
end

do
	local function Callback()
		plugin:ApplySettings()
	end
	
	local FontFlagList = {
		["NONE"] = "None",
		["OUTLINE"] = "Outline",
		["THICKOUTLINE"] = "Thick Outline",
		["MONOCHROME"] = "Monochrome",
	}
	
	local VerticalAlignList = {
		["TOP"] = "Top",
		["MIDDLE"] = "Middle",
		["BOTTOM"] = "Bottom",
	}
	
	local HorizontalAlignList = {
		["LEFT"] = "Left",
		["CENTER"] = "Center",
		["RIGHT"] = "Right",
	}

	local TabSelected
	local LastTabSelected

	function plugin:GetGUI()
		self = plugin	
		
		local returnGroup = AceGUI:Create("SimpleGroup")
		returnGroup:SetFullWidth(true)
		returnGroup:SetLayout("Flow")
		
		returnGroup:AddChild(UIF:CheckBox("Enabled", db, "Enabled", Callback, nil, 0.33))
		
		local tgroup = UIF:TabGroup()
		returnGroup:AddChild(tgroup)
		tgroup:SetTabs({
			{text = "Position & Size", value = "PaS"},
			{text = "Colors & Alpha", value = "CaA"},
			{text = "Font", value = "Font"},
		})		
		tgroup:SetCallback("OnGroupSelected", TabSelected)
		tgroup:SelectTab(LastSelectedTab or "PaS")
		
		return returnGroup
	end
	
	TabSelected = function(self, event, tab)
		self:ReleaseChildren()
		LastSelectedTab = tab
		
		if tab == "PaS" then
			self:AddChild(UIF:Text1("Positions here are relative to the mRunes main position"))
			self:AddChild(UIF:PositionSlider("x", "X", db, "x", 0.1, Callback, 0.5))
			self:AddChild(UIF:PositionSlider("y", "Y", db, "y", 0.1, Callback, 0.5))
			self:AddChild(UIF:Slider("Width", db, "Width", 5, 1000, 0.1, Callback, 0.5))
			self:AddChild(UIF:Slider("Height", db, "Height", 5, 200, 0.1, Callback, 0.5))	
			self:AddChild(UIF:Slider("Space between the bars", db, "BarSpacing", 0, 100, 0.1, Callback, 0.5))
		
		elseif tab == "CaA" then
			self:AddChild(UIF:ColorSelect(FrostName.. " color", db, "FrostColor", Callback, true, 0.5))		
			self:AddChild(UIF:ColorSelect(PlagueName.. " color", db, "PlagueColor", Callback, true, 0.5))		
			
			self:AddChild(UIF:ColorSelect("Background Color", db, "BackgroundColor", Callback, true, 0.5))
			self:AddChild(UIF:LSMDropdown("statusbar", "Bar Texture", db, "Texture", Callback, 0.5))
			self:AddChild(UIF:ColorSelect("Border color", db, "BorderColor", Callback, true, 0.5))
			self:AddChild(UIF:LSMDropdown("border", "Border texture", db, "BorderTexture", Callback, 0.5))		
			self:AddChild(UIF:NewLine(20))
			self:AddChild(UIF:Slider("Border Width", db, "BorderWidth", 0, 50, 1, Callback, 0.33))
			self:AddChild(UIF:Slider("Border Padding", db, "BorderPadding", 0, 100, 0.1, Callback, 0.33))
			self:AddChild(UIF:Slider("Border Insets", db, "BorderInsets", 0, 50, 1, Callback, 0.33))
			
			group = UIF:InlineGroup2("Alpha")			
			self:AddChild(group)
			group:AddChild(UIF:Slider("In combat", db, "AlphaIncCombat", 0, 1, 0.01, Callback, 0.33))
			group:AddChild(UIF:Slider("Out of combat", db, "AlphaOutCombat", 0, 1, 0.01, Callback, 0.33))
		
		elseif tab == "Font" then
			self:AddChild(UIF:Text1("These settings are for the timer texts"))
			self:AddChild(UIF:LSMDropdown("font", "Font", db, "Font", Callback, 0.5))
			self:AddChild(UIF:Dropdown("Font Flags", db, "FontFlags", Callback, FontFlagList, 0.5))		
			self:AddChild(UIF:ColorSelect("Color", db, "FontColor", Callback, false, 0.2))
			self:AddChild(UIF:Slider("Size", db, "FontSize", 2, 50, 1, Callback, 0.4))
			self:AddChild(UIF:Slider("Spacing", db, "InternalSpacing", 0, 50, 0.1, Callback, 0.4))
			self:AddChild(UIF:Dropdown("Horizontal Align", db, "HorAlign", Callback, HorizontalAlignList, 0.5))
			self:AddChild(UIF:Dropdown("Vertical Align", db, "VerAlign", Callback, VerticalAlignList, 0.5))
			self:AddChild(UIF:CheckBox("Show", db, "ShowText", Callback, "Toggle if the runic power text should be shown.", 0.5))
			self:AddChild(UIF:CheckBox("Always Show", db, "AlwaysShowText", Callback, "Toggle if the timer text should always be visible. If not it will be hidden when the runc power is at 0", 0.5))
		end
		
		self:DoLayout()
		self.parent:DoLayout()
	end
end